==> gitlab-ce-v6.7.4.tar: gitlab-ce-v6.7.4/app/services/projects/create_service.rb <==
module Projects
  class CreateService < BaseService
    def initialize(user, params)
      @current_user, @params = user, params.dup
    end

    def execute
      # get namespace id
      namespace_id = params.delete(:namespace_id)

      # check that user is allowed to set specified visibility_level
      unless Gitlab::VisibilityLevel.allowed_for?(current_user, params[:visibility_level])
        params.delete(:visibility_level)
      end

      # Load default feature settings
      default_features = Gitlab.config.gitlab.default_projects_features

      default_opts = {
        issues_enabled: default_features.issues,
        wiki_enabled: default_features.wiki,
        wall_enabled: default_features.wall,
        snippets_enabled: default_features.snippets,
        merge_requests_enabled: default_features.merge_requests,
        visibility_level: default_features.visibility_level
      }.stringify_keys

      @project = Project.new(default_opts.merge(params))

      # Parametrize path for project
      #
      # Ex.
      #  'GitLab HQ'.parameterize => "gitlab-hq"
      #
      @project.path = @project.name.dup.parameterize unless @project.path.present?


      if namespace_id
        # Find matching namespace and check if it allowed
        # for current user if namespace_id passed.
        if allowed_namespace?(current_user, namespace_id)
          @project.namespace_id = namespace_id
        else
          deny_namespace
          return @project
        end
      else
        # Set current user namespace if namespace_id is nil
        @project.namespace_id = current_user.namespace_id
      end

      @project.creator = current_user

      if @project.save
        unless @project.group
          @project.users_projects.create(
            project_access: UsersProject::MASTER,
            user: current_user
          )
        end

        @project.update_column(:last_activity_at, @project.created_at)

        if @project.import?
          @project.import_start
        else
          GitlabShellWorker.perform_async(
            :add_repository,
            @project.path_with_namespace
          )

        end

        if @project.wiki_enabled?
          begin
            # force the creation of a wiki,
            GollumWiki.new(@project, @project.owner).wiki
          rescue GollumWiki::CouldNotCreateWikiError => ex
            # Prevent project observer crash
            # if failed to create wiki
            nil
          end
        end
      end

      @project
    rescue => ex
      @project.errors.add(:base, "Can't save project. Please try again later")
      @project
    end

    protected

    def deny_namespace
      @project.errors.add(:namespace, "is not valid")
    end

    def allowed_namespace?(user, namespace_id)
      namespace = Namespace.find_by(id: namespace_id)
      current_user.can?(:manage_namespace, namespace)
    end
  end
end

==> gitlab-ce-v7.1.1.tar: gitlab-ce-v7.1.1/app/services/projects/create_service.rb <==
module Projects
  class CreateService < BaseService
    def initialize(user, params)
      @current_user, @params = user, params.dup
    end

    def execute
      @project = Project.new(params)

      # Reset visibility levet if is not allowed to set it
      unless Gitlab::VisibilityLevel.allowed_for?(current_user, params[:visibility_level])
        @project.visibility_level = default_features.visibility_level
      end

      # Parametrize path for project
      #
      # Ex.
      #  'GitLab HQ'.parameterize => "gitlab-hq"
      #
      @project.path = @project.name.dup.parameterize unless @project.path.present?

      # get namespace id
      namespace_id = params[:namespace_id]

      if namespace_id
        # Find matching namespace and check if it allowed
        # for current user if namespace_id passed.
        unless allowed_namespace?(current_user, namespace_id)
          @project.namespace_id = nil
          deny_namespace
          return @project
        end
      else
        # Set current user namespace if namespace_id is nil
        @project.namespace_id = current_user.namespace_id
      end

      @project.creator = current_user

      if @project.save
        log_info("#{@project.owner.name} created a new project \"#{@project.name_with_namespace}\"")
        system_hook_service.execute_hooks_for(@project, :create)

        unless @project.group
          @project.users_projects.create(
            project_access: UsersProject::MASTER,
            user: current_user
          )
        end

        @project.update_column(:last_activity_at, @project.created_at)

        if @project.import?
          @project.import_start
        else
          GitlabShellWorker.perform_async(
            :add_repository,
            @project.path_with_namespace
          )

        end

        if @project.wiki_enabled?
          begin
            # force the creation of a wiki,
            ProjectWiki.new(@project, @project.owner).wiki
          rescue ProjectWiki::CouldNotCreateWikiError => ex
            # Prevent project observer crash
            # if failed to create wiki
            nil
          end
        end
      end

      @project
    rescue => ex
      @project.errors.add(:base, "Can't save project. Please try again later")
      @project
    end

    protected

    def deny_namespace
      @project.errors.add(:namespace, "is not valid")
    end

    def allowed_namespace?(user, namespace_id)
      namespace = Namespace.find_by(id: namespace_id)
      current_user.can?(:create_projects, namespace)
    end
  end
end
